<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');

class JExtranetHelperFile extends JObject {

	public $id;
	protected $_db;
	public $name = '';
	public $path = '';
	public $type = '';
	public $size = '';
	public $hits = 0;
	public $creator = '';
	public $parent;
	
	
	public function __construct( $id ) {
		if ( !$id ) {
			throw new Exception( 'invalid constructor for file');
		}
		$this->id	= $id;
		$this->_db 	= JFactory::getDbo();
		$this->getDetails();
	}
	
	
	protected function getDetails() {
		
		$query = $this->_db->getQuery( true );
		$query->select( 
					$this->_db->quoteName('f_title') . ' AS name,' . 
					$this->_db->quoteName('f_alias') . ' AS path,' . 
					$this->_db->quoteName('f_type') . ' AS type,' .
					$this->_db->quoteName('f_size') . ' AS size,' .
					$this->_db->quoteName('f_created_by') . ' AS creator,' .
					$this->_db->quoteName('f_hits') . ' AS hits'
					);
		$query->from( $this->_db->quoteName('#__extranet_files') );
		$query->where( $this->_db->quoteName('f_id') . ' = ' . $this->id );
		
		$this->_db->setQuery($query);
		$this->setProperties( $this->_db->loadAssoc() );				
	}
	
	
	/*
	 * Method to generate a unique token to be used for direct file download, when a file is shared
	 * 
	 * @return		sha1 encrypted token
	 */
	public function generateDownloadLink() {
		$unique = JFactory::getApplication()->getCfg('secret');
		return sha1( $this->id . $this->path . $this->creator . $unique );
	}


	// return file name from server
	// the name of a file in J!Extranet can be different than the real name of file from server
	// this will also contain the leading "/"
	public function getName() 
	{	
		preg_match('#.*(/.*)$#', $this->path, $matches);
		return $matches[1];
	}


	// updates file path and parent
	public function updateSqlPath()
	{
		$query = $this->_db->getQuery();
		$query->update('#__extranet_files');
		$query->set('f_alias = ' . $this->_db->quote($this->path));
		$query->set('f_parent = ' . $this->parent);
		$query->where('f_id = ' . $this->id);

		$this->_db->setQuery($query);
		$this->_db->query();
	}


	/**
	 * copyIt :: String, String -> Boolean
	 * copy a file from a location to another location
	 */
	public function copyIt($from, $to)
	{	
		if (JFile::move($from, $to))
		{
			return true;
		}
		return false;
	}
}
