<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.filesystem.folder');
jimport('joomla.filesystem.file');

require_once(JPATH_COMPONENT_ADMINISTRATOR . '/helpers/FolderAlert.php');

class JExtranetSync {
	
	protected $files = array();
	protected $folders = array();
	protected $_db;
	protected $path;
    protected $id_path_folders;
    protected $increment_fd = 0;
    protected $increment_f = 0;
    protected static $order;
	
    public function __construct($path = null)
    {
		$this->_db 	= JFactory::getDbo();
        $this->files['db'] = array();
        $this->files['server'] = array();
        $this->folders['db'] = array();
        $this->folders['server'] = array();
        $default = JPATH_COMPONENT_ADMINISTRATOR . '/media/extranet';
        $path = JComponentHelper::getParams('com_jextranet')
                        ->get('defaultUploadLocation', $default);
        $this->path = ($path == '' || $path == '/') ? $default : $path;
	}

    public function getItems($path)
    {
        if (!is_dir($path)) {
            return;
        }
		$iterator = new DirectoryIterator($path);
        $stack = array();
		
		foreach ( $iterator as $fileInfo) {

            if ($fileInfo->isDot() || $fileInfo->isLink() ||
                $fileInfo->getFileName() == 'extranet_ufiles'){
				continue;
			}
			
			$item				= new stdclass;
            $item->path			= str_replace($this->path, '',
                                                $fileInfo->getPathname());
			$item->type			= $fileInfo->getExtension();
			$item->name			= $fileInfo->getBasename('.' . $item->type);
            $item->parent		= str_replace($this->path, '',
                                                $fileInfo->getPath());
			$item->size			= $fileInfo->getSize();

            if ($fileInfo->isDir()) {
                $this->folders['server'][] = $item;
                $stack[] = $fileInfo->getPathName(); 
            } else {
                $this->files['server'][] = $item;
            }
		}

        if (!empty($stack)) {
            foreach ($stack as $dir) {
                $this->getItems($dir);
            }
        }
	}

    protected function getStoredFolders()
    {
        $query = $this->_db->getQuery(true);
        $query->select('fd_id as id, fd_name_ftp as path, fd_name as name, ordering');
        $query->from('#__extranet_folders');
        $this->_db->setQuery($query);
        return $this->_db->loadObjectList();
    }

    protected function getStoredFiles()
    {
        $query = $this->_db->getQuery(true);
        $query->select('f_id as id, f_alias as path, f_title as name, f_size as size');
        $query->from('#__extranet_files');
        $this->_db->setQuery($query);
        return $this->_db->loadObjectList();
    }

    protected function insertNewFolders($folders)
    {
        if (empty($folders)) {
            return;
        }

        $this->_db->lockTable('#__extranet_folders');
        $inserted = array_slice($folders, 0, 100);
        $remain  = array_slice($folders, 100);

        $query = $this->_db->getQuery(true);
        $query->insert('#__extranet_folders');
        $query->columns(array(
            $this->_db->quoteName('fd_id'),
            $this->_db->quoteName('fd_parent'),
            $this->_db->quoteName('fd_name'),
            $this->_db->quoteName('fd_name_ftp'),
            $this->_db->quoteName('fd_description'),
            $this->_db->quoteName('fd_created'),
            $this->_db->quoteName('fd_type'),
            $this->_db->quoteName('fd_alerts'),
            $this->_db->quoteName('published'),
            $this->_db->quoteName('ordering'),
            $this->_db->quoteName('checked_out'),
        ));

        $fd_created = JExtranetHelper::getSystem()->getDateToMySQL();
        foreach ($inserted as $folder) {
            $this->increment_fd += 1;
            $fd_parent = $this->getParentId($folder->parent);
            $fd_name_ftp = $folder->path;
            $this->id_path_folders[$this->increment_fd] = $fd_name_ftp;
            $query->values(
                $this->increment_fd . ',' .
                $fd_parent . ',' .
                $this->_db->quote($folder->name) . ',' .
                $this->_db->quote($fd_name_ftp) . ',' .
                $this->_db->quote('') . ',' .
                $this->_db->quote($fd_created) . ',' .
                $this->_db->quote('pv') . ',' .
                '0,1,'.$folder->order.',0'
            );
        }
        $this->_db->setQuery($query);
        $this->_db->query();
        $this->_db->unlockTables();
        return $this->insertNewFolders($remain);
    }

    protected function deleteStoredFolders($folders)
    {
        if (empty($folders)) {
            return;
        }

        $processed = array_slice($folders, 0, 100);
        $remain = array_slice($folders, 100);
        $ids = array_column($folders, 'id');

        $query = $this->_db->getQuery(true);
        $query->delete();
        $query->from('#__extranet_files');
        $query->where('f_parent IN ('.implode(',', $ids).')');

        $this->_db->setQuery($query);
        $this->_db->query();

        $query->clear();
        $query->delete();
        $query->from('#__extranet_folders');
        $query->where('fd_id IN ('.implode(',', $ids).')');
        $this->_db->setQuery($query);
        $this->_db->query();

        return $this->deleteStoredFolders($remain);
    }

    protected function insertNewFiles($files, $offset = 0)
    {
        $inserted = array_slice($files, $offset, 100);

        if (empty($inserted)) {
            return;
        }

        $query = $this->_db->getQuery(true);
        $query->insert('#__extranet_files');
        $query->columns(array(
            $this->_db->quoteName('f_id'),
            $this->_db->quoteName('f_title'),
            $this->_db->quoteName('f_alias'),
            $this->_db->quoteName('f_type'),
            $this->_db->quoteName('f_parent'),
            $this->_db->quoteName('f_preview'),
            $this->_db->quoteName('f_size'),
            $this->_db->quoteName('f_created'),
            $this->_db->quoteName('f_created_by'),
            $this->_db->quoteName('f_hits'),
            $this->_db->quoteName('f_description'),
            $this->_db->quoteName('f_publish_up'),
            $this->_db->quoteName('f_publish_down'),
            $this->_db->quoteName('published'),
            $this->_db->quoteName('ordering'),
            $this->_db->quoteName('checked_out')
        ));
        $f_created = JExtranetHelper::getSystem()->getDateToMySQL();
        $f_created_by = 0;
        foreach ($inserted as $file) {
            $this->increment_f += 1;
            $f_parent = $this->getParentId($file->parent);
            $query->values(
                $this->increment_f . ',' .
                $this->_db->quote($file->name) . ',' .
                $this->_db->quote($file->path) . ',' .
                $this->_db->quote($file->type) . ',' .
                $this->_db->quote($f_parent) . ',' .
                $this->_db->quote('') . ',' .
                $file->size . ',' .
                $this->_db->quote($f_created) . ',' .
                $f_created_by . ',' .
                '0,' .
                $this->_db->quote('') . ',' .
                $this->_db->quote('0000-00-00 00:00:00') . ',' .
                $this->_db->quote('0000-00-00 00:00:00') . ',' .
                '1,'.$file->order.',0'
            );
        }
        $this->_db->setQuery($query);
        $this->_db->query();
        return $this->insertNewFiles($files, $offset+100);
    }

    protected function deleteStoredFiles($files, $offset = 0)
    {
        $processed = array_slice($files, $offset, 100);
        if (empty($processed)) {
            return;
        }

        $ids = array_column($processed, 'id');

        $query = $this->_db->getQuery(true);
        $query->delete();
        $query->from('#__extranet_files');
        $query->where('f_id IN ('.implode(',',$ids).')');
        $this->_db->setQuery($query);
        $this->_db->query();

        return $this->deleteStoredFiles($files, $offset+100);
    }

    protected function differences($items, $size_compare = false)
    {
        $diff = array('minus' => array(), 'plus' => array());

        if (!empty($items)) {
            $values = array_column($items['db'], 'path', 'id');

            foreach ($items['server'] as $fizic) {
                if (!in_array($fizic->path, $values)) {
                    $fizic->order = $this->getOrder($fizic->parent);
                    $diff['plus'][] = $fizic;
                } else {
                    foreach ($items['db'] as $k => $item) {
                        if ($item->path == $fizic->path) {
                            $key = $k;
                            if ($size_compare == true) {
                                if ($fizic->size != $item->size) {
                                    $fizic->order =
                                        $this->getOrder($fizic->parent);
                                    $diff['plus'][] = $fizic;
                                    $diff['minus'][] = $item;
                                } 
                            }
                            break;
                        }
                    }
                    unset($items['db'][$key]);
                }
            }

            if (!empty($items['db'])) {
                foreach ($items['db'] as $item) {
                    $diff['minus'][] = $item;
                }
            }
        }

        return $diff;
    }

    protected function getParentId($path)
    {
        $keys = array_keys($this->id_path_folders, $path);
        if (!empty($keys[0])) {
            return $keys[0];
        }
        return 0;
    }

    protected function getOrder($path) {
        if (!isset(self::$order[$path])) {
            self::$order[$path] = 0;
            $filtered = array_filter($this->folders['db'],
                function($folder) use ($path){
                    return $folder->path == $path;
                }
            );
            if (!empty($filtered)) {
                self::$order[$path] = max(array_column($filtered, 'ordering'));
            }
        }
        
        self::$order[$path] += 1;
        return self::$order[$path];
    }

    public function run()
    {
        $start = time();
        $this->getItems($this->path);
        $this->folders['db'] = $this->getStoredFolders();
        $this->files['db'] = $this->getStoredFiles();

        $differences = array(
            'folders' => $this->differences($this->folders),
            'files' => $this->differences($this->files, true)
        );
        $this->id_path_folders =
            array_column($this->folders['db'], 'path', 'id');
        $this->increment_f = empty($this->files['db']) ? 0 :
            max(array_column($this->files['db'], 'id', 'path'));
        $this->increment_fd = empty($this->id_path_folders) ? 0 :
            max(array_keys($this->id_path_folders)); 

        $folder_plus_count = count($differences['folders']['plus']);
        $folder_minus_count = count($differences['folders']['minus']);
        $files_plus_count = count($differences['files']['plus']);
        $files_minus_count = count($differences['files']['minus']);

        echo "\n";
        printf("(+) New folders on server: %s", $folder_plus_count);

        echo "\n";
        printf("(+) New files on server: %s", $files_plus_count);

        echo "\n";
        printf("(-) Not found folders on server: %s", $folder_minus_count);

        echo "\n";
        printf("(-) Not found files on server: %s", $files_minus_count);

        if ($folder_plus_count > 0) {
            echo "\n";
            echo "Saving new folders to database...";
            $this->insertNewFolders($differences['folders']['plus']);
            echo "Done.";
        }

        if ($files_plus_count > 0) {
            echo "\n";
            echo "Saving new files to database...";
            $this->insertNewFiles($differences['files']['plus']);
            echo "Done.";
        }

        if ($folder_minus_count > 0) {
            echo "\n";
            echo "Delete not found folders from database...";
            $this->deleteStoredFolders($differences['folders']['minus']);
            echo "Done.";
        }

        if ($files_minus_count > 0) {
            echo "\n";
            echo "Delete not found files from database...";
            $this->deleteStoredFiles($differences['files']['minus']);
            echo "Done.";
        }
        $end = time();
        echo "\nFinished in " . ($end - $start) . " seconds.\n";
        exit();
    }
    
    public function silent()
    {
        $this->getItems($this->path);
        $this->folders['db'] = $this->getStoredFolders();
        $this->files['db'] = $this->getStoredFiles();

        $differences = array(
            'folders' => $this->differences($this->folders),
            'files' => $this->differences($this->files, true)
        );
        $this->id_path_folders =
            array_column($this->folders['db'], 'path', 'id');
        $this->increment_f = empty($this->files['db']) ? 0 :
            max(array_column($this->files['db'], 'id', 'path'));
        $this->increment_fd = empty($this->id_path_folders) ? 0 :
            max(array_keys($this->id_path_folders)); 

        $folder_plus_count = count($differences['folders']['plus']);
        $folder_minus_count = count($differences['folders']['minus']);
        $files_plus_count = count($differences['files']['plus']);
        $files_minus_count = count($differences['files']['minus']);

        if ($folder_plus_count > 0) {
            $this->insertNewFolders($differences['folders']['plus']);
        }

        if ($files_plus_count > 0) {
            $this->insertNewFiles($differences['files']['plus']);
        }

        if ($folder_minus_count > 0) {
            $this->deleteStoredFolders($differences['folders']['minus']);
        }

        if ($files_minus_count > 0) {
            $this->deleteStoredFiles($differences['files']['minus']);
        }
    }

}
