<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class JExtranetModelFolders extends JModelList
{
	protected	$option 		= 'COM_JEXTRANET';
	
	public function __construct($config = array()) {
		if (empty($config['filter_fields'])) {
			$config['filter_fields'] = array(
				'a.fd_id',
				'a.fd_name',
				'a.fd_created',
				'a.fd_type',
				'a.published',
				'a.ordering',
			);
		}
		parent::__construct($config);
	}
	
	
	protected function populateState( $ordering = null, $direction = null ) { 
		
		// Initialise variables.
		$app = JFactory::getApplication('administrator');
		
		// Load the filter state.
		$search = $app->getUserStateFromRequest($this->context.'.filter.published', 'filter_published');
		$this->setState('filter.published', $search);
		
		$state = $app->getUserStateFromRequest($this->context.'.filter.type', 'filter_type', '', 'string');
		$this->setState('filter.type', $state);
		
		$state = $app->getUserStateFromRequest($this->context.'.filter.search', 'filter_search', '', 'string');
		$this->setState('filter.search', $state);
		
		// Load the parameters.
		$params = JComponentHelper::getParams('com_jextranet');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('a.fd_id', 'desc');
	}
	
	protected function getStoreId($id = '') {
		
		// Compile the store id.
		$id	.= ':'.$this->getState('filter.published');
		$id	.= ':'.$this->getState('filter.type');
		$id	.= ':'.$this->getState('filter.search');

		return parent::getStoreId($id);
	}
	
	

	protected function getListQuery() {
		
		// Create a new query object.
		$query	= $this->_db->getQuery(true);

		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select','a.fd_id,a.fd_name,a.fd_created,a.fd_type,a.published,a.ordering'
			)
		);
		$query->from( $this->_db->quoteName('#__extranet_folders') . ' AS a' );
		
		$query->where( $this->_db->quoteName('a.fd_parent') . '= 0' );

		// Filter by published state
		$published = $this->getState('filter.published');
		if (is_numeric($published)) {
			$query->where('a.published = '.(int) $published);
		} else if ($published === '') {
			$query->where('(a.published IN (0, 1))');
		}
		
		
		// Filter by search in title
		$search = $this->getState('filter.search');
		if (!empty($search))
		{
			if (stripos($search, 'id:') === 0) {
				$query->where('a.fd_id = '.(int) substr($search, 3));
			}
			else
			{
				$search = $this->_db->Quote('%'.$this->_db->escape($search, true).'%');
				$query->where('( a.fd_name LIKE '.$search.' )');
			}
		}
		
		// Filter by type
		$type = $this->getState('filter.type');
		if (!empty($type)) {
			$type = $this->_db->Quote($this->_db->escape($type, true));
			$query->where('( a.fd_type = '.$type.' )');
		}
		
		
		// Add the list ordering clause.
		$orderCol = $this->getState('list.ordering', 'a.fd_id');
		$query->order($this->_db->escape($orderCol).' '.$this->_db->escape($this->getState('list.direction', 'DESC')));

		return $query;
	}
	
	
	public function getOrdering() {
		
		$query = $this->_db->getQuery(true);
		$query->select( $this->_db->quoteName('a.ordering') );
		$query->from( $this->_db->quoteName('#__extranet_folders') .' AS a');
		$query->where( $this->_db->quoteName('a.fd_parent') . ' = 0' );
		$query->order( $this->_db->quoteName('a.ordering') . ' ASC' );
		$this->_db->setQuery($query);
		$ordering = $this->_db->loadColumn();

		return $ordering;
	}
	
}
?>
