<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */

// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

class JExtranetModelUser extends JModelAdmin
{

	protected $text_prefix = 'COM_JEXTRANET_USERS';
    protected static $injected;
    protected $per_page = 50;

	public function getTable($type = 'Exuser', $prefix = 'JExtranetTable', $config = array())
	{
		return JTable::getInstance($type, $prefix, $config);
	}

	public function getForm($data = array(), $loadData = true)
	{
		// Get the form.
		$form = $this->loadForm('com_jextranet.user', 'user', array('control' => 'jform', 'load_data' => $loadData));
		if (empty($form)) {
			return false;
		}

		return $form;
	}

	protected function loadFormData()
	{
		// Check the session for previously entered form data.
		$data = JFactory::getApplication()->getUserState('com_jextranet.edit.user.data', array());
        $data = $this->injectUserData($data);

		return $data;
	}

	protected function prepareTable( $table ) {

		$params 	= json_decode($table->u_params);
		$form 		= JRequest::getVar('jform', array(), '', 'array');
		$params->allowUserUploadOwnFolder	= $form['allowUserUploadOwnFolder'];
		$params->firstPageContent 			= $form['firstPageContent'];
        for ($i = 0; $i < 4; $i++) {
            $key0 = 'customPage' . $i . 'Status';
            $key1 = 'customPage' . $i;
            $params->$key0 = isset($form[$key0]) ? $form[$key0] : 0;
            $params->$key1 = isset($form[$key1]) ? $form[$key1] : '';
        }

		$table->u_params = json_encode( $params );
	}


	public function donew($users, $blocked = 1, $permissions = array(), $group = 0)
	{
		// if we need to allow him to access everything
		if ( JExtranetHelper::getSystem()->params->get('initialAuthoriseAll', 0) ) {
			$permissions['all'] = array('2');
		}

		$params['permissions'] = $permissions;
		$u_params = json_encode($params);

		$query = $this->_db->getQuery ( true );
		$columns = array( $this->_db->quoteName('u_id'), $this->_db->quoteName('g_id'), $this->_db->quoteName('block'), $this->_db->quoteName('u_params') );
		$values = array();

		foreach ($users as $user)
		{
			if ($user)
			{
				$values[] =  $user . ',' . $group . ',' . $blocked . ', ' . $this->_db->quote($u_params);
			}
		}

		$query->insert($this->_db->quoteName('#__extranet_users'));
		$query->columns($columns);
		$query->values($values);
		$this->_db->setQuery($query);
		$this->_db->query();
	}



	/*
	 * Method to authorise an user to access the extranet
	 *
	 * name: 		unblock ()
	 * @param		array $ids		the id for the users to be authorised
	 * @return
	 */
	public function unblock ( $ids ) {

		$table = $this->getTable();

		foreach ($ids as $id) {
			$id = (int) $id;
			if ($table->load( $id )) {
				$table->block = 1;
				$table->store();
			}
		}
	}



	/*
	 * Method to unauthorise an user to access the extranet
	 *
	 * name: 		unblock ()
	 * @param		array $ids		the id for the users to be unauthorised
	 * @return
	 */
	public function block ( $ids ) {

		$table = $this->getTable();

		foreach ($ids as $id) {
			$id = (int) $id;
			if ($table->load( $id )) {
				$table->block = 0;
				$table->store();
			}
		}
	}

    public function getCustomPages()
    {
        $form = $this->getCustomPagesForm();

        if ($form) {
            $data = $this->loadFormData();
            $form->bind($data);
        }

        return $form;
    }

    public function getCustomPagesForm()
    {
        $form = $this->loadForm(
            'com_jextranet.user_custom_pages',
            'custompages',
            array('control' => 'jform')
        );

		if (empty($form)) {
			return false;
		}
		return $form;
    }

    /*
     * retrieve an object containing a list of items and nr of pages
     * @param $parentFolder     the id of folder ("all" for all folders)
     * @param $offset           view page number (from pagination)
     * @param $user             the user ID to calculate permissions
     * @param $refresh
     *              * since vue will refresh all items during a response,
     *              if we are in the "All" area, permissions div would be
     *              hidden, since the data refresh doesn't maintain the state of
     *              permissions view
     *              * set to 1 to keep open permissions div of "all"
     */
    public function permissionsJsonData($parentFolder, $offset, $user, $refresh = 0)
    {
        $data = new stdclass;
        $items = array();
        $folders = $this->getFolders($parentFolder, $offset);
        $ex_user = JExtranetHelper::getUser($user);

        if(!empty($folders)) {
            if ($parentFolder == 0) {
                $items = array_slice($folders,$offset*$this->per_page,
                                                        $this->per_page);

                // if we are on main folders page, page nr 1
                if ($offset == 0) {
                    $root = new stdclass;
                    $root->id = 'all';
                    $root->name = JText::_('COM_JEXTRANET_ALL');
                    $root->active = $refresh;
                    array_unshift($items, $root);
                }
            } else {
                $items = $folders;
            }
        }

        foreach ($items as &$item) {
            $item->active = (isset($item->active)) ? $item->active : 0;
            $item->permissions['his'] = $ex_user->getOwnPermissions($item->id);
            $item->permissions['group'] = $ex_user->getGroupPermissions($item->id);
            $item->permissions['mixed'] = $ex_user->mixPermissions(
                $item->permissions['his'], $item->permissions['group']
            );
        }
        $data->pages = ceil(count($folders)/$this->per_page);
        $data->items = json_encode($items);

        return $data;
    }

    protected function getFolders($parent = 0)
    {
        $query = $this->_db->getQuery(true);
        $query->select('fd_id AS id,fd_name AS name');
        $query->from('#__extranet_folders');
        $query->where('fd_parent = ' . $parent);
        $query->where('fd_type = '. $this->_db->quote('pv'));
        $query->where('published = 1');
        $query->order('ordering DESC');

        $this->_db->setQuery($query);
        return $this->_db->loadObjectList();
    }

    protected function injectUserData($data)
    {
        if (!isset(self::$injected)) {
            if (empty($data)) {
                $data = $this->getItem();

                // we need to set some data if the user has recordings about if he is allowed to upload
                $user = JTable::getInstance('Exuser','JExtranetTable');
                $user->load($data->u_id);
                $params = json_decode($user->u_params);

                $data->allowUserUploadOwnFolder =
                    isset($params->allowUserUploadOwnFolder) ? $params->allowUserUploadOwnFolder : 0;
                $data->firstPageContent 		=
                    isset($params->firstPageContent) ? $params->firstPageContent : null;

                for ($i = 0; $i < 4; $i++) {
                    $key0 = 'customPage' . $i . 'Status';
                    $key1 = 'customPage' . $i;
                    $data->$key0 = isset($params->$key0) ? $params->$key0 : 0;
                    $data->$key1 = isset($params->$key1) ? $params->$key1 : '';
                }
            }
            self::$injected = $data;
        }

        return self::$injected;
    }

}
