<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

jimport('joomla.filesystem.file');
jimport('joomla.filesystem.folder');
jimport('joomla.client.helper');


class JExtranetModelUserfiles extends JModelList
{
	protected	$option 		= 'COM_JEXTRANET';
	protected 	$file;
	protected	$base;
	
	public function __construct($config = array()) {
		
		$this->base 	= JExtranetHelper::getSystem()->getUploadFolder();
		$this->uid		= JRequest::getVar('id','','','int');
		parent::__construct($config);
		
	}
	
	protected function populateState(   $ordering = null, $direction = null   ) { 
		
		// Load the parameters.
		$params = JComponentHelper::getParams('com_jextranet');
		$this->setState('params', $params);

		// List state information.
		parent::populateState('a.f_id', 'desc');
	}
	
	protected function getStoreId($id = '') {

		return parent::getStoreId($id);
	}
	
	
	public function getPath() {
		$path = $this->base . DIRECTORY_SEPARATOR . 'extranet_ufiles' . DIRECTORY_SEPARATOR . $this->uid . DIRECTORY_SEPARATOR;
		return $path;
	}

	public function getUser() {
		return $this->uid;
	}

	/*
	 * 
	 * name: getListQuery
	 * @param
	 * @return
	 */
	protected function getListQuery() {
		
		// Create a new query object.
		$query	= $this->_db->getQuery(true);
		
		$uid	= JRequest::getVar('id','','GET','int');

		// Select the required fields from the table.
		$query->select(
			$this->getState(
				'list.select','a.f_id,a.f_title,a.f_alias, a.f_type, a.f_size, a.f_created_by '
			)
		);
		$query->from(' #__extranet_users_files AS a');
		
		$query->where('a.f_created_by  = ' . $uid );
		
		// Add the list ordering clause.
		$orderCol = $this->getState('list.ordering', 'a.f_id');
		$query->order($this->_db->escape($orderCol).' '.$this->_db->escape($this->getState('list.direction', 'DESC')));

		return $query;
	}


	public function getFile ( $fileid ) {
		
		clearstatcache();


		$this->file = new JObject();
		$this->file->f_id = 0;
		
		if ( $fileid ) {

			$query = $this->_db->getQuery( true );
			$query->select('a.f_id, a.f_title, a.f_alias, a.f_size, a.f_type');
			$query->from('#__extranet_users_files AS a');
			$query->where('a.f_id = ' . $fileid);
			$this->_db->setQuery($query);
			$file = $this->_db->loadObject();

			if (!empty($file)) {

				$this->file = $file;
				$this->file->location = $this->base . DIRECTORY_SEPARATOR . 'extranet_ufiles' . DIRECTORY_SEPARATOR . $this->uid . DIRECTORY_SEPARATOR . $this->file->f_alias;

				if (JFile::exists( $this->file->location )) { 
					if (function_exists('mime_content_type')) {
						$this->file->mime = mime_content_type($this->file->location);
					} elseif (function_exists('finfo_open')) {
						$finfo 		= finfo_open(FILEINFO_MIME);
						$mimeType 	= @finfo_file($finfo, $this->file->location);
						finfo_close($finfo);
						if (strpos($mimeType, ';')) { 
							list($mimeType,) = explode(';', $mimeType);
							$this->file->mime = $mimeType;
						} 
					} elseif (!empty($mime_type[strtolower($this->file->f_type)])) {
						$this->file->mime = $mime_type[strtolower($this->file->f_type)];
					} else {
						$this->file->mime = 'application/octet-stream';
					}
				}
			}
		} 
		return $this->file; 
	}

	public function getHeaders() {

		header("Pragma: public");
		header("Expires: 0");
		header("Cache-Control: must-revalidate, post-check=0, pre-check=0");
		header("Cache-Control: public");
		header("Content-Description: File Transfer");
		header("Content-Type: " . $this->file->mime);
		if ($this->file->f_size) {
			header("Content-Length: " .(string)($this->file->f_size) );
		}
		header('Content-Disposition: attachment; filename="' . $this->file->f_title . '.' . $this->file->f_type .'"');
		header("Content-Transfer-Encoding: binary\n");
	}


	// method to get a file content
	public function getContent() {
		ob_clean();
		flush();
		$data = JFile::read( $this->file->location );
		echo $data;
		ob_flush();
		flush(); 
		exit(0); 
	}


	
	public function addFileDb( $file ) {
		
		$query = $this->_db->getQuery( true );
		$columns = array( 
				$this->_db->quoteName('f_id'),
				$this->_db->quoteName('f_title'),
				$this->_db->quoteName('f_alias'),
				$this->_db->quoteName('f_type'),
				$this->_db->quoteName('f_size'),
				$this->_db->quoteName('f_created'),
				$this->_db->quoteName('f_created_by'),
				$this->_db->quoteName('f_hits'),
				$this->_db->quoteName('f_description')
				);
		$values = 
				'NULL, ' . 
				$this->_db->Quote( $file->f_title ) . ',' . 
				$this->_db->Quote( $file->f_alias ). ',' . 
				$this->_db->Quote( $file->f_type ). ',' . 
				$file->f_size . ',' . 
				$this->_db->Quote($file->f_created) . ',' . 
				$file->f_created_by . 
				',0,' . 
				$this->_db->Quote('');

		$query->insert('#__extranet_users_files');
		$query->columns($columns);
		$query->values($values);
		$this->_db->setQuery($query);
		$this->_db->query();
	}
	

	/**
	 * addMessage :: Object($notif) -> Void
	 * insert a notification message into db 
	 */
	public function addMessage($notif) 
	{
		$query = $this->_db->getQuery(true);
		$to = json_encode(array($notif->file->f_created_by));
		$date = JFactory::getDate('now')->format('Y-m-d H:i:s');

		$columns = array( 
				$this->_db->quoteName('m_id'),
				$this->_db->quoteName('m_user_id_from'),
				$this->_db->quoteName('m_user_id_to'),
				$this->_db->quoteName('m_date_time'),
				$this->_db->quoteName('published'),
				$this->_db->quoteName('m_subject'),
				$this->_db->quoteName('m_message'),
				$this->_db->quoteName('checked_out'),
				);
		$values = 'NULL,'.$notif->sender->id.','.$this->_db->Quote($notif->file->f_created_by).','. $this->_db->Quote($date).',0,'.$this->_db->Quote($notif->subject).','.$this->_db->Quote($notif->body).',0';

		$query->insert('#__extranet_messages');
		$query->columns($columns);
		$query->values($values);
		$this->_db->setQuery($query);
		$this->_db->query();
	}
	
}
?>
