<?php
/*
 * @component J!Extranet
 * @website : ionutlupu.me
 * @copyright Copyright (C) 2011 Ionut Lupu. All rights reserved.
 * @license : http://www.gnu.org/copyleft/gpl.html GNU/GPL , see license.txt
 */
 
 
// Check to ensure this file is included in Joomla!
defined('_JEXEC') or die('Restricted access');

require_once( 'jextranet.user.php' );

class JExtranetHelperAuthorise {
	
	protected $user;
	protected $view;
	protected $params;

	public function __construct() {
		$this->user 	= JExtranetHelperUser::getInstance();
		$this->view		= JRequest::getCmd('view','jextranet');
		$this->params	= JComponentHelper::getParams('com_jextranet');
	}



	/**
	 * method to return authorisation level based on the view
	 * 1 = no access, 2 = view, 3 = view & download, 4 = view, download, upload
	 **/
	public function getAuthorisationLevel() {
		
		// if he has direct download return true
		$direct = JFactory::getApplication()->getUserState('direct.download.id',0);
		$view	= JRequest::getVar('view');
		
		if ( $direct && $view == 'file' ) {
			return true;
		}
		
		// first let's see if he is logged in  
		// if he isn't logged in let's return 1 = no access
		if ( !$this->user->get('id') ) {
			return false;
		}

		// if restricted IP
		if ( !$this->IPAuthorised() ){
			return false;
		}

		// check if it's an extranet user
		if ( !$this->user->isExtranetUser() ) {
			return false;
		}

		// let's get authorisation level for the folder
		if ( $this->view == 'folders' ) {
			
			$folder = JRequest::getVar('fid', 0, '', 'int');
			// if it's in the root return a value bigger than 1; 1=no rights
			if ( !$folder ) {
				return true;
			}
			$level = $this->user->getAuthorisationLevel ($folder);
	
			if (in_array(1, $level)) {
				return false;
			}
		}


		// get authorisation for other sections

		$removed =  $this->params->get('userMenuItemsRemoved', array() ); 

		// authorisation level for messages
		if ( $this->view == 'messages' ) {
			if ( in_array(2, $removed) ) {
				return false;
			} 
		}

		// authorisation level for bookmarks
		if ( $this->view == 'bookmarks' ) {
			if ( in_array(3, $removed) ) {
				return false;
			} 
		}


		// authorisation level for my vault
		if ( $this->view == 'userfiles' ) {
			if ( in_array(4, $removed) ) {
				return false;
			} 
		}

		return true;
	}




	public function IPAuthorised() {
		
		$ip 		= $this->user->getIP();
		$denied 	= $this->params->get('loginDenyFilter');
		$allowed	= $this->params->get('loginAllowFilter');

		// if we are using a list of DENIED IP's we search and check if the ip is in the list
		if ( $denied ) {
			$list = explode('/', $denied);
			foreach ( $list as $value ) {
				if (stripos( $value, 'x' ) !== false ) {
					$value = str_replace(array('x','X'), '[0-9]{1,3}', $value);
					$regex = '/' . $value . '/i';
					if (preg_match($regex,$ip,$matches)) {
  						return false;
					}
				} else {
					if ( $value == $ip ) {
						return false;
					}
				}
			}
		}

		// if we are using a list of ALLOWED IP's we check if it's in that list
		if ( $allowed ) {
			$list = explode('/', $allowed);
			foreach ( $list as $value ) {
				if (stripos( $value, 'x' ) !== false ) {
					$value = str_replace(array('x','X'), '[0-9]{1,3}', $value);
					$regex = '/' . $value . '/i';
					if (preg_match($regex,$ip,$matches)) {
  						return true;
					}
				} else {
					if ( $value == $ip ) {
						return true;
					}
				}
			}
			return false;
		}

		return true;
	}


	
	
}
?>
